unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ComCtrls, ExtCtrls, StdCtrls, Buttons, ShellAPI, ReadBDEIni;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    OpenINIFile1: TMenuItem;
    CloseINIFile1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    View1: TMenuItem;
    INIFileAsText1: TMenuItem;
    OpenIni: TOpenDialog;
    Panel1: TPanel;
    Panel2: TPanel;
    TView: TTreeView;
    Label1: TLabel;
    lblValue: TLabel;
    Label2: TLabel;
    edtNewValue: TEdit;
    N2: TMenuItem;
    RefreshTreeView1: TMenuItem;
    ImageList1: TImageList;
    Save1: TMenuItem;
    SaveAs1: TMenuItem;
    N3: TMenuItem;
    SaveAsIni: TSaveDialog;
    SB: TStatusBar;
    procedure FormResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure OpenINIFile1Click(Sender: TObject);
    procedure CloseINIFile1Click(Sender: TObject);
    procedure INIFileAsText1Click(Sender: TObject);
    procedure RefreshTreeView1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure TViewChange(Sender: TObject; Node: TTreeNode);
    procedure TViewChanging(Sender: TObject; Node: TTreeNode;
      var AllowChange: Boolean);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
    CurrentNode: TTreeNode;
    BDEIni: TSetupBDE;
    Viewer: string;
    ChangeOK: boolean;
  public
    { Public declarations }
    procedure WMGetMinMaxInfo(var Msg: TWMGetMinMaxInfo);
    		message wm_GetMinMaxInfo;

  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.WMGetMinMaxInfo(var Msg: TWMGetMinMaxInfo);
begin
  Msg.MinMaxInfo^.ptMinTrackSize := point(490, 210);
  inherited;
end;


procedure TMainForm.FormResize(Sender: TObject);
begin
  edtNewValue.Width := Width - (Panel1.Width + 38);
  lblValue.Width := edtNewValue.Width;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  ChangeOK := False;
  lblValue.Caption := '';
  edtNewValue.Text := '';
  BDEIni := nil;
  Viewer := 'notepad.exe';

  try
    BDEIni := TSetupBDE.Create(ExtractFilePath(Application.ExeName) + 'default.ini', TView);
    BDEIni.BDEConfigOnly := True;
    BDEIni.Refresh;
    Caption := 'Network BDE Administration Tool (' + ExtractFilePath(Application.ExeName) +
                'default.ini' + ')';
    ChangeOK := True;
  except
    SB.SimpleText := 'Default file not found.';
  end;
end;

procedure TMainForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.OpenINIFile1Click(Sender: TObject);
begin
  if OpenIni.InitialDir = '' then
    OpenIni.InitialDir := ExtractFilePath(Application.ExeName);
  if OpenIni.Execute = True then
  begin
    OpenIni.InitialDir := ExtractFilePath(OpenIni.FileName);
    if BDEIni = nil then
    begin
      BDEIni := TSetupBDE.Create(OpenIni.FileName, TView);
      BDEIni.Refresh;
      Save1.Enabled := True;
    end
    else
    begin
      if BDEIni.GetIniFileName <> OpenIni.FileName then
        Save1.Enabled := True;

      BDEIni.Free;
      BDEIni := nil;
      ChangeOK := False;
      BDEIni := TSetupBDE.Create(OpenIni.FileName, TView);
      BDEIni.Refresh;
    end;
    ChangeOK := True;
    Caption := 'Network BDE Administration Tool (' + OpenIni.FileName + ')';
  end;
end;


procedure TMainForm.CloseINIFile1Click(Sender: TObject);
begin
  TView.Items.Clear;
  Caption := 'Network BDE Administration Tool';
  if BDEIni <> nil then
  begin
    if BDEIni.IniFileChanged = True then
    begin
      if  MessageDlg('Save INI file', mtConfirmation,
            mbYesNoCancel, 0) = mrYes then
        BDEIni.SaveIniFile;
    end;
    BDEIni.Free;
    BDEIni := nil;
  end;
  ChangeOK := False;
end;

procedure TMainForm.INIFileAsText1Click(Sender: TObject);
begin
  if ShellExecute(0, 'open', PChar(Viewer), PChar(BDEIni.GetTempFileName),
        PChar(ExtractFilePath(BDEIni.GetTempFileName)), SW_SHOWNORMAL) <= 32 then
  begin
    Viewer := 'wordpad.exe';
    if ShellExecute(0, 'open', PChar(Viewer), PChar(BDEIni.GetTempFileName),
        PChar(ExtractFilePath(BDEIni.GetTempFileName)), SW_SHOWNORMAL) <= 32 then
      ShowMessage('Could not find a viewer for the INI file');
  end;

end;

procedure TMainForm.RefreshTreeView1Click(Sender: TObject);
begin
  if BDEIni <> nil then
    BDEIni.Refresh;
end;

procedure TMainForm.Save1Click(Sender: TObject);
var
  Dummy: boolean;
begin
  Dummy := True;
  if CurrentNode <> nil then
    if CurrentNode.Level = 1 then
    begin
      TView.OnChanging(self, CurrentNode, Dummy);
      lblValue.Caption := edtNewValue.Text;
    end;
  BDEIni.SaveIniFile;
end;

procedure TMainForm.SaveAs1Click(Sender: TObject);
var
 Dummy: boolean;
 IniFileName: string;

begin
  SaveAsIni.InitialDir := ExtractFilePath(IniFileName);
  if SaveAsIni.Execute = True then
  begin
    if CurrentNode <> nil then
      if CurrentNode.Level = 1 then
      begin
        TView.OnChanging(self, CurrentNode, Dummy);
        lblValue.Caption := edtNewValue.Text;
      end;
    IniFileName := SaveAsIni.FileName;
    BDEIni.SaveIniFileAs(IniFileName);
    Caption := 'Network BDE Administration Tool (' + IniFileName + ')';
    Save1.Enabled := True;
  end;
end;

procedure TMainForm.TViewChange(Sender: TObject; Node: TTreeNode);
begin
  if TView.Selected.Level = 0 then
  begin
    lblValue.Caption := '< No Item Selected>';
    edtNewValue.Text := '';
    edtNewValue.Enabled := False;
  end;
  if TView.Selected.Level = 1 then
  begin
    Label1.Caption := 'Value for ' + TView.Selected.Text + ':';
    lblValue.Caption := BDEIni.GetAsString(TView.Selected.Parent.Text, TView.Selected.Text);
    edtNewValue.Text := lblValue.Caption;
    edtNewValue.Enabled := True;
  end;
end;

procedure TMainForm.TViewChanging(Sender: TObject; Node: TTreeNode;
  var AllowChange: Boolean);
var
  NewValue: string;

begin
  if (ChangeOK = True) and (TView.Selected <> nil)  then
  begin
    CurrentNode := TView.Selected;
    if CurrentNode.Level = 1 then
    begin
      NewValue := edtNewValue.Text;
      if BDEIni.Update(CurrentNode, NewValue) = true then
        SB.SimpleText := CurrentNode.Text + ' updated to ' + NewValue;
    end;
   end;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  rslt: word;

begin
  if BDEIni = nil then
  begin
    CanClose := True;
    Exit;
  end;

  if edtNewValue.Focused then
  begin
    rslt := MessageDlg('Save new value to INI file', mtConfirmation,
          mbYesNoCancel, 0);
    case rslt of
      mrYes:
      begin
        BDEIni.Update(TView.Selected, edtNewValue.Text);
        CanClose := True;
      end;
      mrNo:
        CanClose := True;
      mrCancel:
        CanClose := False;
    end;
  end;
  if BDEIni.IniFileChanged = True then
  begin
    rslt := MessageDlg('Save INI file', mtConfirmation,
          mbYesNoCancel, 0);
    case rslt of
      mrYes:
      begin
        if Save1.Enabled = True then
          BDEIni.SaveIniFile
        else
          SaveAs1Click(self);
        CanClose := True;
      end;
      mrNo:
        CanClose := True;
      mrCancel:
        CanClose := False;
    end;
  end;
end;

end.
